# Extension for TWiki Enterprise Collaboration Platform, http://TWiki.org/
#
# Copyright (C) 2014-2015 Alba Power Quality Solutions
# Copyright (C) 2010-2016 TWiki Contributors
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read LICENSE in the root of this distribution.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# As per the GPL, removal of this notice is prohibited.

package TWiki::Contrib::MailInContrib::CharSetFilter;
use TWiki::Contrib::MailInContrib::Filter;
our @ISA = qw( TWiki::Contrib::MailInContrib::Filter );

use strict;
use TWiki;
use Assert;

sub process {
    my $this = shift;

    my $charset = lc( $this->contentType() );
    $charset =~ s/.*charset=([a-z0-9\-]*).*/$1/s;
    #$_[0] .= "\n\nDEBUG charset: $charset\n";

    $_[0] =~ s/\r//gs;                              # remove carriage return
    $_[0] =~ s/\n +(?:\n)/\n/gs;                    # remove empty lines that contain only spaces
    $_[0] =~ s/\n\n+/\n\n/gs;                       # reduce multiple empty lines to one
    $_[0] =~ s/\n([>][> ]*\n)*([>][> ]*\n)/\n$2/gs; # reduce multiple empty quoted lines to one
    $_[0] =~ s/\n\n+$/\n/gs;                        # remove newlines at end of text
    $_[0] =~ s/\n([0-9]+\.)  +/\n   $1 /gs;         # \n1. bullet
    $_[0] =~ s/\n(   +)- /\n$1\* /gs;               # \n   - bullet
    $_[0] =~ s/\n *\xC2?\xB7 *\n+/\n   * /gs;       # \n \n(mid-dot) \n bullet
    $_[0] =~ s/(\%)([A-Z]+[\%\{])/$1<nop>$2/gs;     # defuse TWiki variables %XYX% and %XYZ{...}%
    $_[0] =~ s/([^\n])\n([^\|> \n])/$1 %BR%\n$2/gs; # add line break if next line has leading character
    $_[0] =~ s/\n(\-\-\-+ Forward)/\n<nop>$1/gis;   # ---- Forwarded is not horizontal rule

    if( $charset =~ /utf-8|windows-1252|iso-8859-1/ ) {
        $_[0] =~ s/\xc2\xae/&reg;/go;
        $_[0] =~ s/\xc2\xab/&laquo;/go;
        $_[0] =~ s/\xc2\xbb/&raquo;/go;
        $_[0] =~ s/\xc3\x80/&Agrave;/go;
        $_[0] =~ s/\xc3\x81/&Aacute;/go;
        $_[0] =~ s/\xc3\x82/&Acirc;/go;
        $_[0] =~ s/\xc3\x83/&Atilde;/go;
        $_[0] =~ s/\xc3\x84/&Auml;/go;
        $_[0] =~ s/\xc3\x85/&Aring;/go;
        $_[0] =~ s/\xc3\x86/&AElig;/go;
        $_[0] =~ s/\xc3\x87/&Ccedil;/go;
        $_[0] =~ s/\xc3\x88/&Egrave;/go;
        $_[0] =~ s/\xc3\x89/&Eacute;/go;
        $_[0] =~ s/\xc3\x8a/&Ecirc;/go;
        $_[0] =~ s/\xc3\x8b/&Euml;/go;
        $_[0] =~ s/\xc3\x8c/&Igrave;/go;
        $_[0] =~ s/\xc3\x8d/&Iacute;/go;
        $_[0] =~ s/\xc3\x8e/&Icirc;/go;
        $_[0] =~ s/\xc3\x8f/&Iuml;/go;
        $_[0] =~ s/\xc3\x90/&ETH;/go;
        $_[0] =~ s/\xc3\x91/&Ntilde;/go;
        $_[0] =~ s/\xc3\x92/&Ograve;/go;
        $_[0] =~ s/\xc3\x93/&Oacute;/go;
        $_[0] =~ s/\xc3\x94/&Ocirc;/go;
        $_[0] =~ s/\xc3\x95/&Otilde;/go;
        $_[0] =~ s/\xc3\x96/&Ouml;/go;
        $_[0] =~ s/\xc3\x97/&times;/go;
        $_[0] =~ s/\xc3\x98/&Oslash;/go;
        $_[0] =~ s/\xc3\x99/&Ugrave;/go;
        $_[0] =~ s/\xc3\x9a/&Uacute;/go;
        $_[0] =~ s/\xc3\x9b/&Ucirc;/go;
        $_[0] =~ s/\xc3\x9c/&Uuml;/go;
        $_[0] =~ s/\xc3\x9d/&Yacute;/go;
        $_[0] =~ s/\xc3\x9e/&THORN;/go;
        $_[0] =~ s/\xc3\x9f/&szlig;/go;
        $_[0] =~ s/\xc3\xa0/&agrave;/go;
        $_[0] =~ s/\xc3\xa1/&aacute;/go;
        $_[0] =~ s/\xc3\xa2/&acirc;/go;
        $_[0] =~ s/\xc3\xa3/&atilde;/go;
        $_[0] =~ s/\xc3\xa4/&auml;/go;
        $_[0] =~ s/\xc3\xa5/&aring;/go;
        $_[0] =~ s/\xc3\xa6/&aelig;/go;
        $_[0] =~ s/\xc3\xa7/&ccedil;/go;
        $_[0] =~ s/\xc3\xa8/&egrave;/go;
        $_[0] =~ s/\xc3\xa9/&eacute;/go;
        $_[0] =~ s/\xc3\xaa/&ecirc;/go;
        $_[0] =~ s/\xc3\xab/&euml;/go;
        $_[0] =~ s/\xc3\xac/&igrave;/go;
        $_[0] =~ s/\xc3\xad/&iacute;/go;
        $_[0] =~ s/\xc3\xae/&icirc;/go;
        $_[0] =~ s/\xc3\xaf/&iuml;/go;
        $_[0] =~ s/\xc3\xb0/&eth;/go;
        $_[0] =~ s/\xc3\xb1/&ntilde;/go;
        $_[0] =~ s/\xc3\xb2/&ograve;/go;
        $_[0] =~ s/\xc3\xb3/&oacute;/go;
        $_[0] =~ s/\xc3\xb4/&ocirc;/go;
        $_[0] =~ s/\xc3\xb5/&otilde;/go;
        $_[0] =~ s/\xc3\xb6/&ouml;/go;
        $_[0] =~ s/\xc3\xb7/&divide;/go;
        $_[0] =~ s/\xc3\xb8/&oslash;/go;
        $_[0] =~ s/\xc3\xb9/&ugrave;/go;
        $_[0] =~ s/\xc3\xba/&uacute;/go;
        $_[0] =~ s/\xc3\xbb/&ucirc;/go;
        $_[0] =~ s/\xc3\xbc/&uuml;/go;
        $_[0] =~ s/\xc3\xbd/&yacute;/go;
        $_[0] =~ s/\xc3\xbe/&thorn;/go;
        $_[0] =~ s/\xc3\xbf/&yuml;/go;
        $_[0] =~ s/\xe2\x80\xa2/&bull;/go;
        $_[0] =~ s/\xe2\x80\xa6/&hellip;/go;
        $_[0] =~ s/\xe2\x80[\x90\x91\x92\x93\x94]/-/go; # en-dash, utf-8 e28093, x2013
        $_[0] =~ s/\xe2\x80[\x98\x99\x9a\x9b]/'/go;
        $_[0] =~ s/\xe2\x80[\x9c\x9d\x9e\x9f]/"/go;
        $_[0] =~ s/([\xc2\xc3])([\x80-\xbf])/chr(ord($1)<<6&0xC0|ord($2)&0x3F)/eg;
    }

    if( $charset =~ /windows-1252|iso-8859-1/ ) {
        $_[0] =~ s/\x80/&euro;/go;
        $_[0] =~ s/\x82/&sbquo;/go;
        $_[0] =~ s/\x83/&fnof;/go;
        $_[0] =~ s/\x84/&bdquo;/go;
        $_[0] =~ s/\x85/&hellip;/go;
        $_[0] =~ s/\x86/&dagger;/go;
        $_[0] =~ s/\x87/&Dagger;/go;
        $_[0] =~ s/\x88/&circ;/go;
        $_[0] =~ s/\x89/&permil;/go;
        $_[0] =~ s/\x8a/&Scaron;/go;
        $_[0] =~ s/\x8b/&lsaquo;/go;
        $_[0] =~ s/\x8c/&OElig;/go;
        $_[0] =~ s/\x8e/&Zcaron;/go;
        $_[0] =~ s/\x91/&lsquo;/go;
        $_[0] =~ s/\x92/&rsquo;/go;
        $_[0] =~ s/\x93/&ldquo;/go;
        $_[0] =~ s/\x94/&rdquo;/go;
        $_[0] =~ s/\x95/&bull;/go;
        $_[0] =~ s/\x96/&ndash;/go;
        $_[0] =~ s/\x97/&mdash;/go;
        $_[0] =~ s/\x98/&tilde;/go;
        $_[0] =~ s/\x99/&trade;/go;
        $_[0] =~ s/\x9a/&scaron;/go;
        $_[0] =~ s/\x9b/&rsaquo;/go;
        $_[0] =~ s/\x9c/&oelig;/go;
        $_[0] =~ s/\x9e/&zcaron;/go;
        $_[0] =~ s/\x9f/&Yuml;/go;
        $_[0] =~ s/\xa0/&nbsp;/go;
        $_[0] =~ s/\xa1/&iexcl;/go;
        $_[0] =~ s/\xa2/&cent;/go;
        $_[0] =~ s/\xa3/&pound;/go;
        $_[0] =~ s/\xa4/&curren;/go;
        $_[0] =~ s/\xa5/&yen;/go;
        $_[0] =~ s/\xa6/&brvbar;/go;
        $_[0] =~ s/\xa7/&sect;/go;
        $_[0] =~ s/\xa8/&uml;/go;
        $_[0] =~ s/\xa9/&copy;/go;
        $_[0] =~ s/\xaa/&ordf;/go;
        $_[0] =~ s/\xab/&laquo;/go;
        $_[0] =~ s/\xac/&not;/go;
        $_[0] =~ s/\xad/&shy;/go;
        $_[0] =~ s/\xae/&reg;/go;
        $_[0] =~ s/\xaf/&macr;/go;
        $_[0] =~ s/\xb0/&deg;/go;
        $_[0] =~ s/\xb1/&plusmn;/go;
        $_[0] =~ s/\xb2/&sup2;/go;
        $_[0] =~ s/\xb3/&sup3;/go;
        $_[0] =~ s/\xb4/&acute;/go;
        $_[0] =~ s/\xb5/&micro;/go;
        $_[0] =~ s/\xb6/&para;/go;
        $_[0] =~ s/\xb7/&middot;/go;
        $_[0] =~ s/\xb8/&cedil;/go;
        $_[0] =~ s/\xb9/&sup1;/go;
        $_[0] =~ s/\xba/&ordm;/go;
        $_[0] =~ s/\xbb/&raquo;/go;
        $_[0] =~ s/\xbc/&frac14;/go;
        $_[0] =~ s/\xbd/&frac12;/go;
        $_[0] =~ s/\xbe/&frac34;/go;
        $_[0] =~ s/\xbf/&iquest;/go;
        $_[0] =~ s/\xc0/&Agrave;/go;
        $_[0] =~ s/\xc1/&Aacute;/go;
        $_[0] =~ s/\xc2/&Acirc;/go;
        $_[0] =~ s/\xc3/&Atilde;/go;
        $_[0] =~ s/\xc4/&Auml;/go;
        $_[0] =~ s/\xc5/&Aring;/go;
        $_[0] =~ s/\xc6/&AElig;/go;
        $_[0] =~ s/\xc7/&Ccedil;/go;
        $_[0] =~ s/\xc8/&Egrave;/go;
        $_[0] =~ s/\xc9/&Eacute;/go;
        $_[0] =~ s/\xca/&Ecirc;/go;
        $_[0] =~ s/\xcb/&Euml;/go;
        $_[0] =~ s/\xcc/&Igrave;/go;
        $_[0] =~ s/\xcd/&Iacute;/go;
        $_[0] =~ s/\xce/&Icirc;/go;
        $_[0] =~ s/\xcf/&Iuml;/go;
        $_[0] =~ s/\xd0/&ETH;/go;
        $_[0] =~ s/\xd1/&Ntilde;/go;
        $_[0] =~ s/\xd2/&Ograve;/go;
        $_[0] =~ s/\xd3/&Oacute;/go;
        $_[0] =~ s/\xd4/&Ocirc;/go;
        $_[0] =~ s/\xd5/&Otilde;/go;
        $_[0] =~ s/\xd6/&Ouml;/go;
        $_[0] =~ s/\xd7/&times;/go;
        $_[0] =~ s/\xd8/&Oslash;/go;
        $_[0] =~ s/\xd9/&Ugrave;/go;
        $_[0] =~ s/\xda/&Uacute;/go;
        $_[0] =~ s/\xdb/&Ucirc;/go;
        $_[0] =~ s/\xdc/&Uuml;/go;
        $_[0] =~ s/\xdd/&Yacute;/go;
        $_[0] =~ s/\xde/&THORN;/go;
        $_[0] =~ s/\xdf/&szlig;/go;
        $_[0] =~ s/\xe0/&agrave;/go;
        $_[0] =~ s/\xe1/&aacute;/go;
        $_[0] =~ s/\xe2/&acirc;/go;
        $_[0] =~ s/\xe3/&atilde;/go;
        $_[0] =~ s/\xe4/&auml;/go;
        $_[0] =~ s/\xe5/&aring;/go;
        $_[0] =~ s/\xe6/&aelig;/go;
        $_[0] =~ s/\xe7/&ccedil;/go;
        $_[0] =~ s/\xe8/&egrave;/go;
        $_[0] =~ s/\xe9/&eacute;/go;
        $_[0] =~ s/\xea/&ecirc;/go;
        $_[0] =~ s/\xeb/&euml;/go;
        $_[0] =~ s/\xec/&igrave;/go;
        $_[0] =~ s/\xed/&iacute;/go;
        $_[0] =~ s/\xee/&icirc;/go;
        $_[0] =~ s/\xef/&iuml;/go;
        $_[0] =~ s/\xf0/&eth;/go;
        $_[0] =~ s/\xf1/&ntilde;/go;
        $_[0] =~ s/\xf2/&ograve;/go;
        $_[0] =~ s/\xf3/&oacute;/go;
        $_[0] =~ s/\xf4/&ocirc;/go;
        $_[0] =~ s/\xf5/&otilde;/go;
        $_[0] =~ s/\xf6/&ouml;/go;
        $_[0] =~ s/\xf7/&divide;/go;
        $_[0] =~ s/\xf8/&oslash;/go;
        $_[0] =~ s/\xf9/&ugrave;/go;
        $_[0] =~ s/\xfa/&uacute;/go;
        $_[0] =~ s/\xfb/&ucirc;/go;
        $_[0] =~ s/\xfc/&uuml;/go;
        $_[0] =~ s/\xfd/&yacute;/go;
        $_[0] =~ s/\xfe/&thorn;/go;
        $_[0] =~ s/\xff/&yuml;/go;
    }
}

1;
